SemwidgQL.AbstractSyntaxTree = function() {
	//Namespace
};

SemwidgQL.AbstractSyntaxTree.queryToAst = function(semwidgQuery, throwExceptions) {

	try {

		var cstream = new org.antlr.runtime.ANTLRStringStream(semwidgQuery);
		var lexer = new SemwidgQLLexer(cstream);
		var tstream = new org.antlr.runtime.CommonTokenStream(lexer);
		var parser = new SemwidgQLParser(tstream);
		
		var ast = parser.semwidg().tree;

		return ast;

	}catch(e) {
		if (throwExceptions) {
			throw e;
		}else {
			console.error(e);
		}
	}

};

SemwidgQL.AbstractSyntaxTree.astToString = function(ast) {
	var output = '';
	var level = 0;

	output = SemwidgQL.AbstractSyntaxTree._astToStringRecursive(ast, level, '--');

	return output;
};

SemwidgQL.AbstractSyntaxTree._astToStringRecursive = function(tree, level, indentSymbol) {

	var output = '';

	for (var i = 0; i < level; i++) {
		output += indentSymbol;
	}

	if (!(tree.getText() === SemwidgQL.Util.getTypeLabel(tree))) {
		output += '[' + SemwidgQL.Util.getTypeLabel(tree) + '] ' + tree.getText() + '\n';
	}else {
		output += '[' + tree.getText() + ']' + '\n';
	}

	if (tree.getChildren() != null) {
		for (var j = 0; j < tree.getChildren().length; j++) {
			output += SemwidgQL.AbstractSyntaxTree._astToStringRecursive(tree.getChild(j), level + 1, '--');
		}
	}

	return output;
};

SemwidgQL.AbstractSyntaxTree.astToHtml = function(id, ast) {

	if (!ast) {
		return null;
	}

	var output = '';

	var root = SemwidgQL.AbstractSyntaxTree._astToHtmlRecursive(null, ast);

	output += '<ul id="' + id + '">';

	output += root.toHtmlString();

	output += '</ul>'

	return output;
};

SemwidgQL.AbstractSyntaxTree._astToHtmlRecursive = function(parent, child) {

	if (!child) {
		return null;
	}

	var node = new SemwidgQL.AbstractSyntaxTree.AstNodeElement();
	var label = '';
	var length = 12;
	var suffix = '...';

	if (!(child.getText() === SemwidgQL.Util.getTypeLabel(child))) {
		label += '<span title="' 
		label += SemwidgQL.Util.getTypeLabel(child) + '\n';
		label += child.getText();
		label += '">[';
		label += SemwidgQL.Util.truncate(SemwidgQL.Util.getTypeLabel(child), length, suffix);
		label += ']<br>'
		label += SemwidgQL.Util.truncate(child.getText(), length, suffix);
		label += '</span>';
	}else {
		label += '<span title="' + child.getText() + '">['; 
		label += SemwidgQL.Util.truncate(child.getText(), length, suffix);
		label += ']</span>';
	}
	node.setLabel(label);

	if (child.getChildren() != null) {
		for (var j = 0; j < child.getChildren().length; j++) {
			SemwidgQL.AbstractSyntaxTree._astToHtmlRecursive(node, child.getChild(j));
		}
	}

	if (parent) {
		parent.addChild(node);
	}else {
		return node;
	}
};

SemwidgQL.AbstractSyntaxTree.AstNodeElement = function() {
	this.label = '';
	this.children = new Array();
};

SemwidgQL.AbstractSyntaxTree.AstNodeElement.prototype.getLabel = function() {
	return this.label;
};

SemwidgQL.AbstractSyntaxTree.AstNodeElement.prototype.setLabel = function(label) {
	this.label = label;
};

SemwidgQL.AbstractSyntaxTree.AstNodeElement.prototype.getChildren = function() {
	return this.children;
};

SemwidgQL.AbstractSyntaxTree.AstNodeElement.prototype.addChild = function(child) {
	this.children.push(child);
};

SemwidgQL.AbstractSyntaxTree.AstNodeElement.prototype.toHtmlString = function() {

	var str = '';

	if (this.getChildren().length > 0) {
		str += '<li>' + this.getLabel();

		str += '<ul>';

		for (var i = 0; i < this.getChildren().length; i++) {
			str += this.getChildren()[i].toHtmlString();
		}

		str += '</ul>';

		str += '</li>'
	}else {
		str += '<li>' + this.getLabel() + '</li>';
	}
	
	return str;
};
