SemwidgQL.Sparql.Factory = function() {

};

SemwidgQL.Sparql.Factory.unescape = function(value) {

	// unescape \.
	value = value.replace(/\\./g, '.');
	return value;
};

SemwidgQL.Sparql.Factory.buildQuery = function(simpleQuery) {

	var query = '';

	query += SemwidgQL.Sparql.Factory.buildPrefixPart(simpleQuery);
	query += SemwidgQL.Sparql.Factory.buildSelectPart(simpleQuery);
	query += SemwidgQL.Sparql.Factory.buildWherePart(simpleQuery);
	query += SemwidgQL.Sparql.Factory.buildGroupPart(simpleQuery);

	if (query.indexOf('a owl:Thing') > -1) {
		query = 'PREFIX owl: <http://www.w3.org/2002/07/owl#>\n' + query;
	}

	return query;

};

SemwidgQL.Sparql.Factory.buildPrefixPart = function(simpleQuery) {

	var str = '';

	if (simpleQuery) {

		var prefixes = SemwidgQL.Sparql.getUsedPrefixStrings(simpleQuery.getPointer());
		var usedPrefixObjects = SemwidgQL.Sparql.getUsedPrefixObjects(prefixes);

		if (usedPrefixObjects && usedPrefixObjects.length > 0) {
			for (var i = 0; i < usedPrefixObjects.length; i++) {
				str += 'PREFIX ';
				str += usedPrefixObjects[i].getPrefix();
				str += ':';
				str += ' <' + usedPrefixObjects[i].getUrl() + '>';
				str += '\n';
			}
		}
	}

	str = SemwidgQL.Sparql.Factory.unescape(str);

	return str;
};

SemwidgQL.Sparql.Factory.buildSelectPart = function(simpleQuery) {

	var str = '';

	if (simpleQuery) {

		var hiddenElements = SemwidgQL.Sparql.Factory.getHiddenElements(simpleQuery);

		// If group expressions exist, results must be aggregated
		var groupExpressions = SemwidgQL.Sparql.Factory.getGroupExpressions(simpleQuery);
		var aggregate = groupExpressions.length > 0;

		var resourcesAndProperties = SemwidgQL.Sparql.getUsedResourcesAndProperties(simpleQuery);

		str += 'SELECT DISTINCT ';

		var aggregateFunction = null;

		for (var i = 0; i < resourcesAndProperties.length; i++) {

			var elem = resourcesAndProperties[i];
			var variableName = SemwidgQL.Sparql.convertQueryElementToVariableName(elem);

			var hidden = hiddenElements.indexOf(variableName) >= 0;
			if (hidden) {
				continue;
			}

			aggregateFunction = SemwidgQL.Sparql.Factory.getAggregateExpressions(elem, aggregate);

			var prediacte = SemwidgQL.Sparql.Factory.getPredicate(elem);

			if (elem instanceof SemwidgQL.Element.Resource) {

				if (elem.getValue() == '*') {
					// wildcard
					if (aggregateFunction) {
						str += aggregateFunction + '(';
					}
					str += variableName;
					if (aggregateFunction) {
						str += ') AS ' + variableName;
					}
					str += ' ';
				}else {
					// normal resource
					if (aggregateFunction) {
						str += aggregateFunction + '(';
					}
					str += '(' + elem.getValue() + ')';
					if (aggregateFunction) {
						str += ')';
					}
					str += ' AS ' + variableName + ' ';
				}

			}else {

				if (prediacte) {
					str += prediacte + ' ';
				}

				if (aggregateFunction) {
					str += aggregateFunction + '(';
				}
				str += variableName;
				if (aggregateFunction) {
					str += ') AS ' + variableName;
				}
				str += ' ';
			}

		}

		str += '\n';
	}

	str = SemwidgQL.Sparql.Factory.unescape(str);

	return str;

};

SemwidgQL.Sparql.Factory.getHiddenElements = function(simpleQuery) {

	var hiddenElements = [];
	var hideExpressions = SemwidgQL.Sparql.Factory.getHideExpressions(simpleQuery);
	for (var i = 0; i < hideExpressions.length; i++) {
		if (hideExpressions[i].filter.getHide().toLowerCase() == 'true') {
			hiddenElements.push(SemwidgQL.Sparql.convertQueryElementToVariableName(
					hideExpressions[i].elem));
		}
	}
	return hiddenElements;
};

SemwidgQL.Sparql.Factory.getHideExpressions = function(elem) {
	var arr = [];

	if (!elem) {
		return arr;
	}

	var i = 0;

	if (elem instanceof SemwidgQL.Element.SimpleQuery) {
        arr = arr.concat(SemwidgQL.Sparql.Factory.getHideExpressions(elem.getResource()));
		arr = arr.concat(SemwidgQL.Sparql.Factory.getHideExpressions(elem.getPathExpression()));
	}
	else if (elem instanceof SemwidgQL.Element.PathExpression) {
		if (elem.getMultiplePathExpression()) {
			for (i = 0; i < elem.getMultiplePathExpression().getPathExpressions().length; i++) {
				arr = arr.concat(SemwidgQL.Sparql.Factory.getHideExpressions(elem.getMultiplePathExpression().getPathExpressions()[i]));
			}
		}

		if (elem.getProperties()) {
			for (i = 0; i < elem.getProperties().length; i++) {
				arr = arr.concat(SemwidgQL.Sparql.Factory.getHideExpressions(elem.getProperties()[i]));
			}
		}
	}
	else if (elem instanceof SemwidgQL.Element.Property) {
		if (elem.getFilter()) {
			for (i = 0; i < elem.getFilter().length; i++) {

				var filter = elem.getFilter()[i]
				if (filter instanceof SemwidgQL.Element.KeywordExpression.Hide) {
					arr.push({'elem': elem, 'filter': filter});
				}
			}
		}
	}
    else if (elem instanceof SemwidgQL.Element.Resource) {
        if (elem.getFilter()) {
            for (i = 0; i < elem.getFilter().length; i++) {

                var filter = elem.getFilter()[i]
                if (filter instanceof SemwidgQL.Element.KeywordExpression.Hide) {
                    arr.push({'elem': elem, 'filter': filter});
                }
            }
        }
    }

	return arr;
};

SemwidgQL.Sparql.Factory.getAggregateExpressions = function(elem, forceAggregateExpression) {

	if (elem && elem.getFilter()) {

		var filterExpressions = elem.getFilter();

		for (var i = 0; i < filterExpressions.length; i++) {
			if (filterExpressions[i] instanceof SemwidgQL.Element.KeywordExpression.Aggregate) {
				return (filterExpressions[i].getAggregate()).trim().toUpperCase();
			}
		}
	}

	if (forceAggregateExpression) {
		return 'SAMPLE';
	} else {
		return null;
	}
};

SemwidgQL.Sparql.Factory.getPredicate = function(elem) {

	if (elem && elem.getFilter()) {

		var filterExpressions = elem.getFilter();

		for (var i = 0; i < filterExpressions.length; i++) {
			if (filterExpressions[i] instanceof SemwidgQL.Element.KeywordExpression.Predicate 
				&& filterExpressions[i].getPredicate().toLowerCase() == 'true') {
				
				var value = elem.getValue();
				if (value == '*') {
					value = SemwidgQL.Sparql.convertQueryElementToVariableName(elem);
				}

				var type = elem.getType();

				switch(type) {
				case 'ForwardProperty':
				case 'ForwardPropertyWithFilter':
				case 'ReversedProperty':
				case 'ReversedPropertyWithFilter':
					return '("' + value + '") AS ?PredicateFor' + SemwidgQL.Sparql.convertQueryElementToVariableName(elem).substr(1);
				case 'ForwardWildcard':
				case 'ForwardWildcardWithFilter':
				case 'ReversedWildcard':
				case 'ReversedWildcardWithFilter':
					return '?PredicateFor' + SemwidgQL.Sparql.convertQueryElementToVariableName(elem).substr(1);
				}
			}
		}
	}

	return null;
};

SemwidgQL.Sparql.Factory.getOptional = function(elem) {

	if (elem && elem.getFilter()) {

		var filterExpressions = elem.getFilter();

		for (var i = 0; i < filterExpressions.length; i++) {
			if (filterExpressions[i] instanceof SemwidgQL.Element.KeywordExpression.Optional) {
				return (filterExpressions[i].getOptional()).trim().toLowerCase() == 'true';
			}
		}
	}

	return false;
};

SemwidgQL.Sparql.Factory.buildWherePart = function(simpleQuery) {

	var str = '';

	if (simpleQuery) {

		str += 'WHERE {\n';

		str += SemwidgQL.Sparql.Factory.buildPathExpression(simpleQuery.getResource(), simpleQuery.getPathExpression());
			
		str += '}';

	}

	str = SemwidgQL.Sparql.Factory.unescape(str);

	return str;

};

SemwidgQL.Sparql.Factory.buildPathExpression = function(previousElement, pathExpression) {

	var str = '';
	var optional = false;
	var filter = null;

	if (pathExpression) {

		if (pathExpression.getProperties() && pathExpression.getProperties().length > 0) {

			var prop = pathExpression.getProperties()[0];

			if (previousElement instanceof SemwidgQL.Element.Resource) {

				optional = SemwidgQL.Sparql.Factory.getOptional(prop);
				if (optional) {
					str += '\tOPTIONAL { ##' + SemwidgQL.Sparql.convertQueryElementToVariableName(prop) + ' \n';
				}

				str += SemwidgQL.Sparql.Factory.buildFirstTriple(previousElement, prop);
				str += SemwidgQL.Sparql.Factory.buildFilter(previousElement);

				if (optional) {
					str += '\t} ##' + SemwidgQL.Sparql.convertQueryElementToVariableName(prop) + ' \n';
				}

			}else {

				optional = SemwidgQL.Sparql.Factory.getOptional(prop);
				if (optional) {
					str += '\tOPTIONAL { ##' + SemwidgQL.Sparql.convertQueryElementToVariableName(prop) + ' \n';
				}

				str += SemwidgQL.Sparql.Factory.buildTriple(previousElement, prop);

				if (optional) {
					str += '\t} ##' + SemwidgQL.Sparql.convertQueryElementToVariableName(prop) + ' \n';
				}
			}
			
			str += SemwidgQL.Sparql.Factory.buildFilter(prop);

			for (var i = 0; i < pathExpression.getProperties().length - 1; i++) {

				optional = SemwidgQL.Sparql.Factory.getOptional(pathExpression.getProperties()[i + 1]);
				if (optional) {
					str += '\tOPTIONAL { ##' + SemwidgQL.Sparql.convertQueryElementToVariableName(pathExpression.getProperties()[i + 1]) + ' \n';
				}

				str += SemwidgQL.Sparql.Factory.buildTriple(pathExpression.getProperties()[i], pathExpression.getProperties()[i + 1]);
				str += SemwidgQL.Sparql.Factory.buildFilter(pathExpression.getProperties()[i + 1]);
				
				if (optional) {
					str += '\t} ##' + SemwidgQL.Sparql.convertQueryElementToVariableName(pathExpression.getProperties()[i + 1]) + ' \n';
				}
			}

			if (pathExpression.getMultiplePathExpression() && pathExpression.getMultiplePathExpression().getPathExpressions().length > 0) {
				for (var j = 0; j < pathExpression.getMultiplePathExpression().getPathExpressions().length; j ++) {

					var pathEx = pathExpression.getMultiplePathExpression().getPathExpressions()[j];

					var property = pathExpression.getProperties()[pathExpression.getProperties().length - 1];
					// console.log(property);
					// str += '\tOPTIONAL { \n';
					str += SemwidgQL.Sparql.Factory.buildPathExpression(property, pathEx);
					// str += '\t} \n';
				}
			}

		}else if (pathExpression.getMultiplePathExpression() && pathExpression.getMultiplePathExpression().getPathExpressions().length > 0){
			for (var k = 0; k < pathExpression.getMultiplePathExpression().getPathExpressions().length; k ++) {

				var pathEx = pathExpression.getMultiplePathExpression().getPathExpressions()[k];

				// str += '\tOPTIONAL { \n';
				str += SemwidgQL.Sparql.Factory.buildPathExpression(previousElement, pathEx);
				// str += '\t} \n';

			}
		}
	}else {

		if (previousElement.getType() == 'ForwardWildcard') {
			str += '\t';
			str += SemwidgQL.Sparql.convertQueryElementToVariableName(previousElement);
			str += ' ?p ?o . \n';
		}
		
		str += SemwidgQL.Sparql.Factory.buildFilter(previousElement);
	}

	// remove duplicate lines
	// TODO: Make it cleverer. Currently, parentheses level is not considered.
	var arr = str.split('\n').reverse().reduce(function(previousValue, currentValue, currentIndex, array){
		
		if (previousValue.indexOf(currentValue) < 0 ||
				currentValue.trim() == 'FILTER (' ||
				currentValue.trim() == ') .' ) {
			previousValue.push(currentValue);
		}

		// remove empty filters
		if (previousValue.length > 1 &&
				previousValue[previousValue.length - 1].trim() == 'FILTER (' &&
				previousValue[previousValue.length - 2].trim() == ') .') {

			previousValue.pop();
			previousValue.pop();
		}

		return previousValue;
	}, []);

	var index = -1;
	for (var l = 0; l < arr.length; l++) {
		index = arr[l].indexOf('##');
		if (index > -1) {
			arr[l] = arr[l].substr(0, index);
		}
	}

	str = arr.reverse().join('\n');

	return str;

};

SemwidgQL.Sparql.Factory.buildFirstTriple = function(elem1, elem2) {

	var str = '\t';

	var value1 = elem1.getValue();

	// resource is wildcard
	if (value1 == '*') {
		value1 = SemwidgQL.Sparql.convertQueryElementToVariableName(elem1);
		// str += value1 + ' a owl:Thing . \n\t';
	}

	var type = elem2.getType();

	switch(type) {
	case 'ForwardProperty':
	case 'ForwardPropertyWithFilter':
		str += value1 + ' ';
		str += elem2.getValue() + ' ';
		str += SemwidgQL.Sparql.convertQueryElementToVariableName(elem2) + ' . \n';
		break;
	case 'ReversedProperty':
	case 'ReversedPropertyWithFilter':
		str += SemwidgQL.Sparql.convertQueryElementToVariableName(elem2) + ' ';
		str += elem2.getValue() + ' ';
		str += value1 + ' . \n';
		break;
	case 'ForwardWildcard':
	case 'ForwardWildcardWithFilter':
		// var propetyName = SemwidgQL.Sparql.convertQueryElementToVariableName(elem1);
		str += value1 + ' ';
		// str += '?PredicateFor' + propetyName.substr(1) + '__' + SemwidgQL.Sparql.convertQueryElementToVariableName(elem2).substr(1) + ' ';
		str += '?PredicateFor' + SemwidgQL.Sparql.convertQueryElementToVariableName(elem2).substr(1) + ' ';
		str += SemwidgQL.Sparql.convertQueryElementToVariableName(elem2) + ' . \n';
		break;
	case 'ReversedWildcard':
	case 'ReversedWildcardWithFilter':
		var propetyName = SemwidgQL.Sparql.convertQueryElementToVariableName(elem2);
		str += propetyName + ' ';
		// str += '?PredicateFor' + propetyName.substr(1) + '__' + SemwidgQL.Sparql.convertQueryElementToVariableName(elem1).substr(1) + ' ';
		str += '?PredicateFor' + propetyName.substr(1) + ' ';
		str += value1 + ' . \n';
		break;
	}

	str = SemwidgQL.Sparql.Factory.unescape(str);

	return str;
};

SemwidgQL.Sparql.Factory.buildTriple = function(elem1, elem2) {

	var str = '\t';
	var type = elem2.getType();

	switch(type) {
	case 'ForwardProperty':
	case 'ForwardPropertyWithFilter':
		str += SemwidgQL.Sparql.convertQueryElementToVariableName(elem1) + ' ';
		str += elem2.getValue() + ' ';
		str += SemwidgQL.Sparql.convertQueryElementToVariableName(elem2) + ' .\n';
		break;
	case 'ReversedProperty':
	case 'ReversedPropertyWithFilter':
		str += SemwidgQL.Sparql.convertQueryElementToVariableName(elem2) + ' ';
		str += elem2.getValue() + ' ';
		str += SemwidgQL.Sparql.convertQueryElementToVariableName(elem1) + ' .\n';
		break;
	case 'ForwardWildcard':
	case 'ForwardWildcardWithFilter':
		// var propetyName = SemwidgQL.Sparql.convertQueryElementToVariableName(elem1);
		str += SemwidgQL.Sparql.convertQueryElementToVariableName(elem1) + ' ';
		// str += '?PredicateFor' + propetyName.substr(1) + '__' + SemwidgQL.Sparql.convertQueryElementToVariableName(elem2).substr(1) + ' ';
		str += '?PredicateFor' + SemwidgQL.Sparql.convertQueryElementToVariableName(elem2).substr(1) + ' ';
		str += SemwidgQL.Sparql.convertQueryElementToVariableName(elem2) + ' .\n';
		break;
	case 'ReversedWildcard':
	case 'ReversedWildcardWithFilter':
		var propetyName = SemwidgQL.Sparql.convertQueryElementToVariableName(elem2);
		str += SemwidgQL.Sparql.convertQueryElementToVariableName(elem2) + ' ';
		// str += '?PredicateFor' + propetyName.substr(1) + '__' + SemwidgQL.Sparql.convertQueryElementToVariableName(elem1).substr(1) + ' ';
		str += '?PredicateFor' + propetyName.substr(1) + ' ';
		str += SemwidgQL.Sparql.convertQueryElementToVariableName(elem1) + ' .\n';
		break;
	}

	str = SemwidgQL.Sparql.Factory.unescape(str);

	return str;
};

SemwidgQL.Sparql.Factory.buildFilter = function(property) {

	var str = '';

	if (property.getType() == 'ForwardPropertyWithFilter' ||
			property.getType() == 'ReversedPropertyWithFilter' ||
			property.getType() == 'ForwardWildcardWithFilter' ||
			property.getType() == 'ReversedWildcardWithFilter' &&
			property.getFilter()) {

		var filter = property.getFilter();

		var realFilters = [];
		var pseudoFilters = [];

		// build triples to get values for Filter expression
		// and split into real and pseudo Filter expressions
		for (var i = 0; i < filter.length; i = i + 2) {

			var filterExpression = filter[i];
			str += SemwidgQL.Sparql.Factory.buildFilterTriples(property, filterExpression);


			if (filterExpression instanceof SemwidgQL.Element.KeywordExpression.Timeinterval ||
				filterExpression instanceof SemwidgQL.Element.KeywordExpression.Hide ||
				filterExpression instanceof SemwidgQL.Element.KeywordExpression.Aggregate ||
				filterExpression instanceof SemwidgQL.Element.KeywordExpression.Optional ||
				filterExpression instanceof SemwidgQL.Element.KeywordExpression.Predicate) {
				
				pseudoFilters.push(filterExpression);

			}else {

				if (realFilters.length > 0 && i > 0) {
					realFilters.push(filter[i - 1]);
				}
				realFilters.push(filterExpression);

			}
		}

		// build real Filter expression
		if (realFilters.length > 0) {
			str += '\t';
			str += 'FILTER (\n\t\t';

			for (var j = 0; j < realFilters.length; j = j + 2) {

				var filterExpression = realFilters[j];
				
				str += SemwidgQL.Sparql.Factory.buildFilterExpression(property, filterExpression);

				if (j + 1 < realFilters.length && realFilters[j + 1] instanceof SemwidgQL.Element.ConditionalOperator) {
					str += ' ' + realFilters[j + 1].getValue() + '\n\t\t';
				}
				
			}

			str += '\n\t) .\n';
		}

		// build pseudo Filter expression
		for (var i = 0; i < pseudoFilters.length; i++) {
			if (pseudoFilters[i] instanceof SemwidgQL.Element.KeywordExpression.Timeinterval) {
				str += '\tBIND(FLOOR((xsd:dateTime(';
				str += SemwidgQL.Sparql.convertQueryElementToVariableName(property);
				str += ') - xsd:dateTime("1970-01-01T00:00:00"))';
				str += ' / (' + SemwidgQL.Util.readPeriodExpression(pseudoFilters[i].getTimeinterval()) + ') ) AS ';
				str += SemwidgQL.Sparql.convertQueryElementToVariableName(property) + '_timeinterval)\n';
			}
		}
	}

	str = SemwidgQL.Sparql.Factory.unescape(str);

	return str;
	
};

SemwidgQL.Sparql.Factory.buildFilterTriples = function(property, filterExpression) {

	var str = '';

	if (filterExpression instanceof SemwidgQL.Element.RelationalExpression) {

		var properties = [property].concat(filterExpression.getProperties());
		
		for (var j = 0; j < properties.length - 1; j++) {
			str += SemwidgQL.Sparql.Factory.buildTriple(properties[j], properties[j + 1]);
		}

		var values = filterExpression.getValue();

		for (var i = 0; i < values.length; i++) {
			var value = values[i];

			if (value instanceof SemwidgQL.Element.SimpleQuery) {
				str += SemwidgQL.Sparql.Factory.buildPathExpression(value.getResource(), value.getPathExpression());
			}
		}
		
	}
	else if (filterExpression instanceof SemwidgQL.Element.KeywordExpression.Self) {

		var values = filterExpression.getValue();

		for (var i = 0; i < values.length; i++) {
			var value = values[i];

			if (value instanceof SemwidgQL.Element.SimpleQuery) {
				str += SemwidgQL.Sparql.Factory.buildPathExpression(value.getResource(), value.getPathExpression());
			}
		}
		
	}
	else if (filterExpression instanceof SemwidgQL.Element.KeywordExpression.Type) {
		
		var variableName = SemwidgQL.Sparql.convertQueryElementToVariableName(property);

		str += '\t';
		str += variableName;
		str += ' rdf:type ';
		str += '?TypeOf_' + variableName.substr(1);
		str += ' . \n';
	}

	str = SemwidgQL.Sparql.Factory.unescape(str);

	return str;

};

SemwidgQL.Sparql.Factory.buildFilterExpression = function(property, filterExpression) {

	var str = '';

	if (filterExpression instanceof SemwidgQL.Element.KeywordExpression.Language) {

		str += 'lang(';
			str += SemwidgQL.Sparql.convertQueryElementToVariableName(property);
		str += ') = \'\' || ';
		str += 'langMatches(lang(';
			str += SemwidgQL.Sparql.convertQueryElementToVariableName(property);
			str += '), ';
			str += filterExpression.getLanguage();
		str += ')';

	}
	else if (filterExpression instanceof SemwidgQL.Element.KeywordExpression.Type) {

		var variableName = SemwidgQL.Sparql.convertQueryElementToVariableName(property);
		str += '?TypeOf_' + variableName.substr(1);
		str += ' ' + filterExpression.getOperator() + ' ';
		str += filterExpression.getType();

	}
	else if (filterExpression instanceof SemwidgQL.Element.KeywordExpression.Self) {

		var values = filterExpression.getValue();
		var lastProperties = new Array();
		
		for (var i = 0; i < values.length; i++) {
			var value = values[i];

			if (value instanceof SemwidgQL.Element.SimpleQuery) {

				if (value.getPathExpression()) {
					lastProperties.push(SemwidgQL.Sparql.Factory.getLastPropertiesOfPathExpression(value.getPathExpression()));
				}
				
			}
		}

		if (lastProperties.length == 0) {
			
			// "CONTAINS" check
			if (filterExpression.getOperator() == '~') {
				str += 'CONTAINS( ';
				str += 'LCASE(STR(';
				str += SemwidgQL.Sparql.convertQueryElementToVariableName(property);
				str += ')), LCASE(STR(';

				for (var i = 0; i < values.length; i++) {
					var value = values[i];
					if (value instanceof SemwidgQL.Element.SimpleQuery) {
						str += SemwidgQL.Sparql.convertQueryElementToVariableName(value.getResource());
					}else {
						str += value + ' ';
					}
				}
				str += ')))';

			}else {
				// check if String
				if (values[0].toString().charAt(0) == '"' || values[0].toString().charAt(0) == "'") {
					str += 'STR(';
					str += SemwidgQL.Sparql.convertQueryElementToVariableName(property);
					str += ') ';
				}else {
					str += SemwidgQL.Sparql.convertQueryElementToVariableName(property) + ' ';
				}

				str += filterExpression.getOperator() + ' ';

				for (var i = 0; i < values.length; i++) {
					var value = values[i];
					if (value instanceof SemwidgQL.Element.SimpleQuery) {
						str += value.getResource().getValue() + ' ';
					}else {
						str += value + ' ';
					}
					
				}
			}
		}else {
			var numberOfFilters = 1;

			for (var i = 0; i < lastProperties.length; i++) {
				numberOfFilters *= lastProperties[i].length;
			}
			
			var propertyIndices = new Array();
			var loops = lastProperties.length;
			for (var l = 0; l < loops; l++) {
				propertyIndices.push(0);
			}

			for (var j = 0; j < numberOfFilters; j++) {

				// "CONTAINS" check
				if (filterExpression.getOperator() == '~') {
					str += 'CONTAINS( ';
					str += 'LCASE(STR(';

					str += SemwidgQL.Sparql.convertQueryElementToVariableName(property);
					str += ')), LCASE(';

					var simpleQueryIndex = 0;
					for (var k = 0; k < values.length; k++) {
						var value = values[k];

						if (value instanceof SemwidgQL.Element.SimpleQuery) {
							var simpleQuerysProperties = lastProperties[simpleQueryIndex];
							var simpleQuerysProperty = simpleQuerysProperties[propertyIndices[simpleQueryIndex]];
							simpleQueryIndex++;
							str += SemwidgQL.Sparql.convertQueryElementToVariableName(simpleQuerysProperty) + ' ';
						}else {
							str += value + ' ';
						}
					}

					str += '))';

				}else {

					if (values[0].toString().charAt(0) == '"' || values[0].toString().charAt(0) == "'") {
						str += 'STR(';
						str += SemwidgQL.Sparql.convertQueryElementToVariableName(property);
						str += ') ';
					}else {
						str += SemwidgQL.Sparql.convertQueryElementToVariableName(property) + ' ';
					}

					str += filterExpression.getOperator() + ' ';

					var simpleQueryIndex = 0;
					for (var k = 0; k < values.length; k++) {
						var value = values[k];

						if (value instanceof SemwidgQL.Element.SimpleQuery) {
							var simpleQuerysProperties = lastProperties[simpleQueryIndex];
							var simpleQuerysProperty = simpleQuerysProperties[propertyIndices[simpleQueryIndex]];
							simpleQueryIndex++;
							str += SemwidgQL.Sparql.convertQueryElementToVariableName(simpleQuerysProperty) + ' ';
						}else {
							str += value + ' ';
						}
					}
				}

				// Define last properties indices for next iteration
				var increaseNext = true;
				for (var m = 0; m < loops; m++) {

					var val = propertyIndices[m];
					var length = lastProperties[m].length;

					if (increaseNext){
						propertyIndices[m] = (val + 1) % length;

						if ((val + 1) == propertyIndices[m]) {
							increaseNext = false;
						}else {
							increaseNext = true;
						}
					}
				}

				if (j < numberOfFilters - 1) {
					str += ' || \n\t\t';
				}
			}
		}

	}
	else if (filterExpression instanceof SemwidgQL.Element.RelationalExpression) {

		var values = filterExpression.getValue();
		var lastProperties = new Array();
		
		for (var i = 0; i < values.length; i++) {
			var value = values[i];

			if (value instanceof SemwidgQL.Element.SimpleQuery) {

				if (value.getPathExpression()) {
					lastProperties.push(SemwidgQL.Sparql.Factory.getLastPropertiesOfPathExpression(value.getPathExpression()));
				}
				
			}
		}

		var properties = filterExpression.getProperties();
		
		if (lastProperties.length == 0) {
			
			// "CONTAINS" check
			if (filterExpression.getOperator() == '~') {
				str += 'CONTAINS( ';
				str += 'LCASE(STR(';
				str += SemwidgQL.Sparql.convertQueryElementToVariableName(properties[properties.length - 1]);
				str += ')), LCASE(STR(';

				for (var i = 0; i < values.length; i++) {
					var value = values[i];
					if (value instanceof SemwidgQL.Element.SimpleQuery) {
						str += SemwidgQL.Sparql.convertQueryElementToVariableName(value.getResource());
					}else {
						str += value + ' ';
					}
				}
				str += ')))';

			}else {
				// check if String
				if (values[0].toString().charAt(0) == '"' || values[0].toString().charAt(0) == "'") {
					str += 'STR(';
					str += SemwidgQL.Sparql.convertQueryElementToVariableName(properties[properties.length - 1]);
					str += ') ';
				}else {
					str += SemwidgQL.Sparql.convertQueryElementToVariableName(properties[properties.length - 1]) + ' ';
				}

				str += filterExpression.getOperator() + ' ';

				for (var i = 0; i < values.length; i++) {
					var value = values[i];
					if (value instanceof SemwidgQL.Element.SimpleQuery) {
						str += value.getResource().getValue() + ' ';
					}else {
						str += value + ' ';
					}
					
				}
			}
		}else {
			var numberOfFilters = 1;

			for (var i = 0; i < lastProperties.length; i++) {
				numberOfFilters *= lastProperties[i].length;
			}
			
			var propertyIndices = new Array();
			var loops = lastProperties.length;
			for (var l = 0; l < loops; l++) {
				propertyIndices.push(0);
			}

			for (var j = 0; j < numberOfFilters; j++) {

				// "CONTAINS" check
				if (filterExpression.getOperator() == '~') {
					str += 'CONTAINS( ';
					str += 'LCASE(STR(';

					str += SemwidgQL.Sparql.convertQueryElementToVariableName(properties[properties.length - 1]);
					str += ')), LCASE(';

					var simpleQueryIndex = 0;
					for (var k = 0; k < values.length; k++) {
						var value = values[k];

						if (value instanceof SemwidgQL.Element.SimpleQuery) {
							var simpleQuerysProperties = lastProperties[simpleQueryIndex];
							var simpleQuerysProperty = simpleQuerysProperties[propertyIndices[simpleQueryIndex]];
							simpleQueryIndex++;
							str += SemwidgQL.Sparql.convertQueryElementToVariableName(simpleQuerysProperty) + ' ';
						}else {
							str += value + ' ';
						}
					}

					str += '))';

				}else {

					if (values[0].toString().charAt(0) == '"' || values[0].toString().charAt(0) == "'") {
						str += 'STR(';
						str += SemwidgQL.Sparql.convertQueryElementToVariableName(properties[properties.length - 1]);
						str += ') ';
					}else {
						str += SemwidgQL.Sparql.convertQueryElementToVariableName(properties[properties.length - 1]) + ' ';
					}

					str += filterExpression.getOperator() + ' ';

					var simpleQueryIndex = 0;
					for (var k = 0; k < values.length; k++) {
						var value = values[k];

						if (value instanceof SemwidgQL.Element.SimpleQuery) {
							var simpleQuerysProperties = lastProperties[simpleQueryIndex];
							var simpleQuerysProperty = simpleQuerysProperties[propertyIndices[simpleQueryIndex]];
							simpleQueryIndex++;
							str += SemwidgQL.Sparql.convertQueryElementToVariableName(simpleQuerysProperty) + ' ';
						}else {
							str += value + ' ';
						}
					}
				}

				// Define last properties indices for next iteration
				var increaseNext = true;
				for (var m = 0; m < loops; m++) {

					var val = propertyIndices[m];
					var length = lastProperties[m].length;

					if (increaseNext){
						propertyIndices[m] = (val + 1) % length;

						if ((val + 1) == propertyIndices[m]) {
							increaseNext = false;
						}else {
							increaseNext = true;
						}
					}
				}

				if (j < numberOfFilters - 1) {
					str += ' || \n\t\t';
				}
			}
		}
	}
	else if (filterExpression instanceof SemwidgQL.Element.KeywordExpression.Timestart) {
		str += 'xsd:dateTime(' + SemwidgQL.Sparql.convertQueryElementToVariableName(property) + ')';
		str += ' >= ';
		str += SemwidgQL.Util.readDateExpression(filterExpression.getTimestart());
	}
	else if (filterExpression instanceof SemwidgQL.Element.KeywordExpression.Timeend) {
		str += 'xsd:dateTime(' + SemwidgQL.Sparql.convertQueryElementToVariableName(property) + ')';
		str += ' <= ';
		str += SemwidgQL.Util.readDateExpression(filterExpression.getTimeend());
	}

	str = SemwidgQL.Sparql.Factory.unescape(str);

	return str;

};

SemwidgQL.Sparql.Factory.getLastPropertiesOfPathExpression = function(pathExpression) {

	var lastProperties = new Array();

	if (!pathExpression) {
		return lastProperties;
	}

	if (pathExpression.getMultiplePathExpression()) {
		var multiplePathExpressions = pathExpression.getMultiplePathExpression().getPathExpressions();
		for (var i = 0; i < multiplePathExpressions.length; i++) {
			var multiplePathExpression = multiplePathExpressions[i];
			lastProperties = lastProperties.concat(
						SemwidgQL.Sparql.Factory.getLastPropertiesOfPathExpression(multiplePathExpression));
		}
	}else if (pathExpression.getProperties() && pathExpression.getProperties().length > 0){
		lastProperties.push(pathExpression.getProperties()[pathExpression.getProperties().length - 1]);
	}
	
	return lastProperties;

};

SemwidgQL.Sparql.Factory.buildGroupPart = function(simpleQuery) {

	var str = '';

	var groupExpressions = SemwidgQL.Sparql.Factory.getGroupExpressions(simpleQuery);

	if (groupExpressions.length > 0) {
		var groupExpression;

		for (var i = 0; i < groupExpressions.length; i++) {
			groupExpression = groupExpressions[i];

			if (groupExpression.filter instanceof SemwidgQL.Element.KeywordExpression.Timeinterval) {

				var name = SemwidgQL.Sparql.convertQueryElementToVariableName(groupExpression.elem) + '_timeinterval';

				str += '\nGROUP BY ' + name;
				str += '\nORDER BY DESC(' + name + ')';
			}
		}
	}

	return str;
};

SemwidgQL.Sparql.Factory.getGroupExpressions = function(elem) {
	var arr = [];

	if (!elem) {
		return arr;
	}

	var i = 0;

	if (elem instanceof SemwidgQL.Element.SimpleQuery) {
		arr = arr.concat(SemwidgQL.Sparql.Factory.getGroupExpressions(elem.getPathExpression()));
	}
	else if (elem instanceof SemwidgQL.Element.PathExpression) {
		if (elem.getMultiplePathExpression()) {
			for (i = 0; i < elem.getMultiplePathExpression().getPathExpressions().length; i++) {
				arr = arr.concat(SemwidgQL.Sparql.Factory.getGroupExpressions(elem.getMultiplePathExpression().getPathExpressions()[i]));
			}
		}

		if (elem.getProperties()) {
			for (i = 0; i < elem.getProperties().length; i++) {
				arr = arr.concat(SemwidgQL.Sparql.Factory.getGroupExpressions(elem.getProperties()[i]));
			}
		}
	}
	else if (elem instanceof SemwidgQL.Element.Property) {
		if (elem.getFilter()) {
			for (i = 0; i < elem.getFilter().length; i++) {

				var filter = elem.getFilter()[i];
				if (filter instanceof SemwidgQL.Element.KeywordExpression.Timeinterval) {
					arr.push({'elem': elem, 'filter': filter});
				}
			}
		}
	}

	return arr;
};

