SemwidgQL.Sparql = function() {

};

SemwidgQL.Sparql.createSparqlQueries = function(ast) {

	var sparqlQueries = new Array();
	var simpleQueries = SemwidgQL.Sparql.getSimpleQueries(ast);

	SemwidgQL.Sparql.applyUrisToNamedResources();

	if (simpleQueries) {

		for (var i = 0; i < simpleQueries.length; i++) {

			var simpleQuery = simpleQueries[i];

			var sparqlQuery = SemwidgQL.Sparql.Factory.buildQuery(simpleQuery);

			if (sparqlQuery) {
				sparqlQueries.push(sparqlQuery);
			}
		}
	}

	return sparqlQueries;

};

SemwidgQL.Sparql.getUsedPrefixStrings = function(node, prefixes) {

	if (!node) {
		return null;
	}

	if (!prefixes) {
		prefixes = new Array();
	}

	if (node.getChildren()) {
		for (var i = 0; i < node.getChildren().length; i++) {
			SemwidgQL.Sparql.getUsedPrefixStrings(node.getChild(i), prefixes);
		}
	}

	if (SemwidgQL.Util.checkAstType(node, 'NamespacePrefix')) {
		var prefix = node.getText();
		if (prefix.charAt(prefix.length - 1) == ':') {
			prefix = prefix.substr(0, prefix.length - 1);
		}
		var contains = false;
		for (var i = 0; i < prefixes.length; i++) {
			if (prefixes[i] == prefix) {
				contains = true;
				break;
			}
		}
		if (!contains) {
			prefixes.push(prefix);
		}
	}

	return prefixes;
};

SemwidgQL.Sparql.getUsedPrefixObjects = function(prefixStrings) {

	var pre = new Array();
	var prefixList = SemwidgQL.getCurrentModel().getPrefixList();

	if (prefixStrings && prefixStrings.length > 0 && prefixList && prefixList.length > 0) {
		for (var i = 0; i < prefixStrings.length; i++) {

			var prefixString = prefixStrings[i];

			for (var j = 0; j < prefixList.length; j++) {

				var prefixObject = prefixList[j];

				if (prefixString == prefixObject.getPrefix()) {
					pre.push(prefixObject);
					break;
				}
			}
		}
	}

	return pre;

};

SemwidgQL.Sparql.getUsedResourcesAndProperties = function(simpleQuery) {

	if (!simpleQuery) {
		return null;
	}
	
	var elements = new Array();
	var resource = simpleQuery.getResource();
	elements.push(resource);

	if (resource.getFilter()) {
		var value;
		for (var i = 0; i < resource.getFilter().length; i++) {
			if (resource.getFilter()[i] instanceof SemwidgQL.Element.RelationalExpression) {
				value = resource.getFilter()[i].getValue();
				if (value && value[0] instanceof SemwidgQL.Element.SimpleQuery) {
					elements = elements.concat(SemwidgQL.Sparql.getUsedResourcesAndProperties(value[0]));
				}
			}
		}
	}
	

	var pathExpression = simpleQuery.getPathExpression();
	elements = elements.concat(SemwidgQL.Sparql._getUsesResourcesAndPropertiesFromPathExpression(pathExpression));

	return elements;
};

SemwidgQL.Sparql._getUsesResourcesAndPropertiesFromPathExpression = function(pathExpression) {

	var elements = new Array();

	if (pathExpression) {

		var properties = pathExpression.getProperties();
		if (properties) {
			for (var i = 0; i < properties.length; i++) {
				var property = properties[i];
				elements.push(property);

				if (property.getType() == 'ForwardPropertyWithFilter' ||
					property.getType() == 'ReversedPropertyWithFilter') {

					var filters = property.getFilter();
					for (var j = 0; j < filters.length; j++) {
						var filter = filters[j];
						if (filter instanceof SemwidgQL.Element.RelationalExpression) {
							var values = filter.getValue();
							for (var k = 0; k < values.length; k++) {
								var value = values[k];
								if (value instanceof SemwidgQL.Element.SimpleQuery) {
									elements = elements.concat(SemwidgQL.Sparql.getUsedResourcesAndProperties(value));
								}
							}
						}
					}
				}
			}
		}

		var multiplePathExpression = pathExpression.getMultiplePathExpression();
		if (multiplePathExpression && multiplePathExpression.getPathExpressions() && multiplePathExpression.getPathExpressions().length > 0) {
			for (var j = 0; j < multiplePathExpression.getPathExpressions().length; j++) {
				var multiPathEx = multiplePathExpression.getPathExpressions()[j];
				elements = elements.concat(SemwidgQL.Sparql._getUsesResourcesAndPropertiesFromPathExpression(multiPathEx));
			}
		}
	}

	return elements;

};

SemwidgQL.Sparql.getSimpleQueries = function(node) {

	if (!node) {
		return new Array();
	}

	var simpleQueries = new Array();

	if (!SemwidgQL.Util.checkAstType(node, 'SimpleQuery')) {

		if (node.getChildren()) {
			for (var i = 0; i < node.getChildren().length; i++) {
				simpleQueries = simpleQueries.concat(SemwidgQL.Sparql.getSimpleQueries(node.getChild(i)));
			}
		}

	}else {

		var simpleQuery = new SemwidgQL.Element.SimpleQuery();
		simpleQuery.setPointer(node);

		if (node.getChildren() && node.getChildren().length >= 1 && node.getChild(0)) {
			var resource = SemwidgQL.Sparql.getResource(node.getChild(0));
			simpleQuery.setResource(resource);
		}

		if (node.getChildren() && node.getChildren().length >= 2 && node.getChild(1)) {
			var pathExpression = SemwidgQL.Sparql.getPathExpression(node.getChild(1));
			simpleQuery.setPathExpression(pathExpression);
		}

		simpleQueries.push(simpleQuery);

	}

	return simpleQueries;

};

SemwidgQL.Sparql.getResource = function(node) {
	if (!node) {
		return null;
	}

	if (SemwidgQL.Util.checkAstType(node, 'Resource')) {

		var res = node.getChild(0);
		var type = SemwidgQL.Util.getTypeLabel(res);
		var value = null;
		var filter = null;

		switch (type) {
		case 'NamedResource':
			value = res.getChild(0).getText();
			break;
		case 'PrefixedResource':
			value = res.getChild(0).getText();
			value += res.getChild(1).getText();
			break;
		case 'QualifiedResource':
			value = '<' + res.getChild(0).getText() + '>';
			break;
		case 'ForwardWildcard':
			value = '*';
			break;
		case 'ForwardWildcardWithFilter':
			value = '*';
			filter = SemwidgQL.Sparql.getFilter(res.getChild(0));
			break;
		default:
			console.error('Type "' + type + '" for a Resource is unknown.');
			break;
		}

		if (value) {
			var resource = new SemwidgQL.Element.Resource();
			resource.setType(type);
			resource.setValue(value);
			resource.setPointer(node);
			resource.setFilter(filter);

			SemwidgQL.getCurrentModel().addToAllResourcesList(resource);

			return resource;
		}

	}else {
		console.error('Expected element of type "Resource". Instead, element of type "' + SemwidgQL.Util.getTypeLabel(node[0]) + '" was commited.');
	}

	return null;
};

SemwidgQL.Sparql.getPathExpression = function(node) {
	if (!node) {
		return null;
	}

	if (SemwidgQL.Util.checkAstType(node, 'PathExpression')) {

		var properties = SemwidgQL.Sparql.getProperties(node);
		var multiplePathExpression = SemwidgQL.Sparql.getMultiplePathExpression(node);
	
		if (properties || multiplePathExpression) {
			var pathExpression = new SemwidgQL.Element.PathExpression();
			pathExpression.setProperties(properties);
			pathExpression.setMultiplePathExpression(multiplePathExpression);
			pathExpression.setPointer(node);

			return pathExpression;
		}

	}else {
		console.error('Expected element of type "PathExpression". Instead, element of type "' + SemwidgQL.Util.getTypeLabel(node[0]) + '" was commited.');
	}
};

SemwidgQL.Sparql.getProperties = function(node) {
	if (!node || !node.getChildren()) {
		return null;
	}

	if (SemwidgQL.Util.checkAstType(node, 'PathExpression') || SemwidgQL.Util.checkAstType(node, 'RelationalExpression')) {

		if (node.getChildren()) {

			var children = node.getChildren();
			var properties = new Array();

			for (var i = 0; i < children.length; i++) {
				if (SemwidgQL.Util.checkAstType(children[i], 'Property')) {
					
					var prop = children[i].getChild(0);
					var type = SemwidgQL.Util.getTypeLabel(prop);
					var value = null;
					var filter = null;

					switch (type) {
					case 'ForwardProperty':
						value = SemwidgQL.Util.getPropertyValue(prop.getChild(0));
						break;
					case 'ForwardPropertyWithFilter':
						value = SemwidgQL.Util.getPropertyValue(prop.getChild(0));
						filter = SemwidgQL.Sparql.getFilter(prop.getChild(1));
						break;
					case 'ReversedProperty':
						value = SemwidgQL.Util.getPropertyValue(prop.getChild(0));
						break;
					case 'ReversedPropertyWithFilter':
						value = SemwidgQL.Util.getPropertyValue(prop.getChild(0));
						filter = SemwidgQL.Sparql.getFilter(prop.getChild(1));
						break;
					case 'ForwardWildcard':
						value = '*';
						break;
					case 'ForwardWildcardWithFilter':
						value = '*';
						filter = SemwidgQL.Sparql.getFilter(prop.getChild(0));
						break;
					case 'ReversedWildcard':
						value = '*';
						break;
					case 'ReversedWildcardWithFilter':
						value = '*';
						filter = SemwidgQL.Sparql.getFilter(prop.getChild(0));
						break;
					default:
						console.error('Type "' + type + '" for a Property is unknown.');
						break;
					}

					if (value) {
						var property = new SemwidgQL.Element.Property();
						property.setType(type);
						property.setValue(value);
						property.setFilter(filter);
						property.setPointer(prop);

						SemwidgQL.getCurrentModel().addToAllPropertiesList(property);

						properties.push(property);
					}
				}
			}

			if (properties.length > 0) {
				return properties;
			}

		}

	}else {
		console.error('Expected element of type "PathExpression". Instead, element of type "' + SemwidgQL.Util.getTypeLabel(node) + '" was commited.');
	}

	return null;

};

SemwidgQL.Sparql.getFilter = function(node) {

	if (!node || !node.getChildren()) {
		return null;
	}

	var filter;

	if (SemwidgQL.Util.checkAstType(node, 'ConditionalExpression')) {

		filter = new Array();

		var orExpressions = node.getChild(0);

		var conditionalAnd = new SemwidgQL.Element.ConditionalOperator();
		conditionalAnd.setValue('&&');

		var conditionalOr = new SemwidgQL.Element.ConditionalOperator();
		conditionalOr.setValue('||');

		for (var i = 0; i < orExpressions.getChildren().length; i++) {

			var andExpressions = orExpressions.getChild(i);

			for (var j = 0; j < andExpressions.getChildren().length; j++) {

				var expression = andExpressions.getChild(j);

				filter.push(SemwidgQL.Sparql.getExpression(expression));

				if (j < andExpressions.getChildren().length - 1) {
					filter.push(conditionalAnd);
				}
			}

			if (i < orExpressions.getChildren().length - 1) {
				filter.push(conditionalOr);
			}
		}

	}

	return filter;

};

SemwidgQL.Sparql.getExpression = function(node) {

	if (!node || !node.getChildren()) {
		return null;
	}

	var expression;

	if (SemwidgQL.Util.checkAstType(node, 'Expression')) {

		var type = SemwidgQL.Util.getTypeLabel(node.getChild(0));

		switch (type) {
		case 'KeywordExpression':
			expression = SemwidgQL.Sparql.getKeywordExpression(node.getChild(0));
			break;
		case 'RelationalExpression':
			expression = SemwidgQL.Sparql.getRelationalExpression(node.getChild(0));
			break;
		}

	}

	return expression;
};

SemwidgQL.Sparql.getKeywordExpression = function(node) {

	if (!node || !node.getChildren()) {
		return null;
	}
	
	var keywordExpression;

	if (SemwidgQL.Util.checkAstType(node, 'KeywordExpression')) {

		var type = SemwidgQL.Util.getTypeLabel(node.getChild(0));

		switch (type) {
		case 'Language':
			if (node.getChild(0).getChild(0) && node.getChild(0).getChild(0).getChild(0)) {

				keywordExpression = new SemwidgQL.Element.KeywordExpression.Language();
				keywordExpression.setOperator('=');
				keywordExpression.setLanguage(node.getChild(0).getChild(0).getChild(0).getText());
			}
			break;
		case 'Type':
			if (node.getChild(0).getChild(0) && node.getChild(0).getChild(0).getChild(0)) {

				keywordExpression = new SemwidgQL.Element.KeywordExpression.Type();
				keywordExpression.setOperator(node.getChild(0).getChild(0).getChild(0).getText());
				keywordExpression.setType(SemwidgQL.Util.getPropertyValue(node.getChild(0).getChild(1).getChild(0)));
			}
			break;
		case 'Self':
			if (node.getChild(0).getChild(0) && node.getChild(0).getChild(0).getChild(0) 
					&& node.getChild(0).getChild(1) && node.getChild(0).getChild(1).getChild(0)) {

				keywordExpression = new SemwidgQL.Element.KeywordExpression.Self();
				keywordExpression.setOperator(node.getChild(0).getChild(0).getChild(0).getText());
				keywordExpression.setValue(SemwidgQL.Sparql.getRelationalExpressionValue(node.getChild(0).getChild(1).getChild(0)));
			}
			break;
		case 'Timeinterval':
			if (node.getChild(0).getChild(0) && node.getChild(0).getChild(0).getChild(0)) {

				keywordExpression = new SemwidgQL.Element.KeywordExpression.Timeinterval();
				keywordExpression.setOperator('=');

				var timeinterval = node.getChild(0).getChild(0).getChild(0).getText();
				timeinterval = timeinterval.substr(1, timeinterval.length - 2);
				keywordExpression.setTimeinterval(timeinterval);
			}
			break;
		case 'Timestart':
			if (node.getChild(0).getChild(0) && node.getChild(0).getChild(0).getChild(0)) {

				keywordExpression = new SemwidgQL.Element.KeywordExpression.Timestart();
				keywordExpression.setOperator('=');

				var timestart = node.getChild(0).getChild(0).getChild(0).getText();
				timestart = timestart.substr(1, timestart.length - 2);
				keywordExpression.setTimestart(timestart);
			}
			break;
		case 'Timeend':
			if (node.getChild(0).getChild(0) && node.getChild(0).getChild(0).getChild(0)) {

				keywordExpression = new SemwidgQL.Element.KeywordExpression.Timeend();
				keywordExpression.setOperator('=');

				var timeend = node.getChild(0).getChild(0).getChild(0).getText();
				timeend = timeend.substr(1, timeend.length - 2);
				keywordExpression.setTimeend(timeend);
			}
			break;
		case 'Aggregate':
			if (node.getChild(0).getChild(0) && node.getChild(0).getChild(0).getChild(0)) {

				keywordExpression = new SemwidgQL.Element.KeywordExpression.Aggregate();
				keywordExpression.setOperator('=');

				var aggregate = node.getChild(0).getChild(0).getChild(0).getText();
				aggregate = aggregate.substr(1, aggregate.length - 2);
				keywordExpression.setAggregate(aggregate);
			}
			break;
		case 'Hide':
			if (node.getChild(0).getChild(0) && node.getChild(0).getChild(0).getChild(0)) {

				keywordExpression = new SemwidgQL.Element.KeywordExpression.Hide();
				keywordExpression.setOperator('=');

				var hide = node.getChild(0).getChild(0).getChild(0).getText();
				keywordExpression.setHide(hide);
			}
			break;
		case 'Optional':
			if (node.getChild(0).getChild(0) && node.getChild(0).getChild(0).getChild(0)) {

				keywordExpression = new SemwidgQL.Element.KeywordExpression.Optional();
				keywordExpression.setOperator('=');

				var optional = node.getChild(0).getChild(0).getChild(0).getText();
				keywordExpression.setOptional(optional);
			}
			break;
		case 'Predicate':
			if (node.getChild(0).getChild(0) && node.getChild(0).getChild(0).getChild(0)) {

				keywordExpression = new SemwidgQL.Element.KeywordExpression.Predicate();
				keywordExpression.setOperator('=');

				var predicate = node.getChild(0).getChild(0).getChild(0).getText();
				keywordExpression.setPredicate(predicate);
			}
			break;
		default:
			console.error('Type "' + type + '" for a KeywordExpression is unknown.')
		}

	}

	return keywordExpression;

};

SemwidgQL.Sparql.getRelationalExpression = function(node) {

	if (!node || !node.getChildren()) {
		return null;
	}

	var relationalExpression;

	if (SemwidgQL.Util.checkAstType(node, 'RelationalExpression')) {

		var relationalExpression = new SemwidgQL.Element.RelationalExpression();

		var properties = SemwidgQL.Sparql.getProperties(node);

		relationalExpression.setProperties(properties);

		var operator = node.getChild(node.getChildCount() - 2).getChild(0).getText();

		relationalExpression.setOperator(operator);

		var value = SemwidgQL.Sparql.getRelationalExpressionValue(node.getChild(node.getChildCount() - 1));

		relationalExpression.setValue(value);
	}

	return relationalExpression;

};

SemwidgQL.Sparql.getRelationalExpressionValue = function(node) {
	
	var values = new Array();

	values = values.concat(SemwidgQL.Sparql._getRelationalExpressionValueRecursive(node));

	return values;
};

SemwidgQL.Sparql._getRelationalExpressionValueRecursive = function(node) {

	var values = new Array();

	var type = SemwidgQL.Util.getTypeLabel(node);

	switch (type) {
	case 'SimpleQuery':
		values = values.concat(SemwidgQL.Sparql.getSimpleQueries(node));
		break;
	case 'QualifiedProperty':
	case 'PrefixedProperty':
		var str = ''
		for (var i = 0; i < node.children.length; i++) {
			str += node.getChild(i).getText();
		}
		values.push(str);
		break;
	default:
		if (node.children && node.children.length > 0) {
			for (var i = 0; i < node.children.length; i++) {
				values = values.concat(SemwidgQL.Sparql._getRelationalExpressionValueRecursive(node.getChild(i)));
			}
		}else {
			values.push(node.getText());
		}
		break;
	}

	return values;

};

SemwidgQL.Sparql.getMultiplePathExpression = function(node) {
	if (!node || !node.getChildren()) {
		return null;
	}

	if (SemwidgQL.Util.checkAstType(node, 'PathExpression')) {

		if (node.getChildren()) {

			var lastChild = node.getChild(node.getChildren().length - 1);
			var multiplePathExpression = null;

			if (SemwidgQL.Util.checkAstType(lastChild, 'MultiplePathExpression') && lastChild.getChildren()) {
				
				
				var pathExpressions = new Array();

				for (var i = 0; i < lastChild.getChildren().length; i++) {
					
					var pathExpression = SemwidgQL.Sparql.getPathExpression(lastChild.getChild(i));
					
					if (pathExpression) {

						pathExpressions.push(pathExpression);

					}
				}

				if (pathExpressions.length > 0) {
					multiplePathExpression = new SemwidgQL.Element.MultiplePathExpression();
					multiplePathExpression.setPathExpressions(pathExpressions);
					multiplePathExpression.setPointer(node);

					return multiplePathExpression;
				}

			}

		}

	}else {
		console.error('Expected element of type "PathExpression". Instead, element of type "' + SemwidgQL.Util.getTypeLabel(node) + '" was commited.')
	}

	return null;

};

SemwidgQL.Sparql.applyUrisToNamedResources = function() {

	var resources = SemwidgQL.getCurrentModel().getAllResourcesList();
	var namedResources = SemwidgQL.getCurrentModel().getNamedResourceList();

	if (!resources || !namedResources) {
		return;
	}

	for (var i = 0; i < resources.length; i++) {
		var res = resources[i];

		if (res.getType() == 'NamedResource') {

			var resourceUriApplied = false;

			for (var j = 0; j < namedResources.length; j++) {
				var named = namedResources[j];

				if (named.getName() == res.getValue()) {

					// URI
					if (named.getValue().indexOf('//') >= 0) {
						res.setValue('<' + named.getValue() + '>');
						res.setType('QualifiedResource');
						resourceUriApplied = true;
					// Prefixed Resource
					}else if (named.getValue().indexOf(':') >= 0) {
						res.setValue(named.getValue());
						res.setType('PrefixedResource');
						resourceUriApplied = true;
					}

					break;
				}
			}

			if (!resourceUriApplied) {
				throw { 
					name: 		'Parser Error', 
					message: 	'"' + res.getValue() + '" is not a valid resource name.' ,
					toString: 	function(){return this.name + ": " + this.message;} 
				};
			}
		}
	}
};

SemwidgQL.Sparql.convertQueryElementToVariableName = function(elem) {

	var variableName = '';
	
	if (elem instanceof SemwidgQL.Element.Resource) {

		if (elem.getValue() == '*') {
			variableName += 'wildcard';
		}else {
			variableName += SemwidgQL.Sparql.valueToLabel(elem.getValue());
		}
		variableName += '__R';
		variableName += SemwidgQL.Sparql.treePathToLabelSuffix(elem.getPointer());

	}else if (elem instanceof SemwidgQL.Element.Property) {

		if (elem.getValue() == '*') {
			variableName += 'wildcard';
		}else {
			variableName += SemwidgQL.Sparql.valueToLabel(elem.getValue());
		}

		if (elem.getType() == 'ReversedProperty' || elem.getType() == 'ReversedPropertyWithFilter') {
			variableName += 'Of';
		}

		variableName += '__P';
		variableName += SemwidgQL.Sparql.treePathToLabelSuffix(elem.getPointer());

	}

	variableName = '?' + variableName.replace(/([^a-zA-Z0-9_ ])|^[_0-9]+/g, '_');

	return variableName;

};

SemwidgQL.Sparql.valueToLabel = function(value) {

	var label = '';

	if (value.indexOf('<') == 0) {

		var uri = value.substring(1, value.length - 1);

		// remove trailing slash
		if (uri.lastIndexOf('/') == uri.length - 1) {
			uri = uri.substring(0, uri.length - 1);
		}

		var slash = uri.lastIndexOf('/');
		var hash = uri.lastIndexOf('#');
		label = uri.substr(Math.max(slash, hash) + 1);
		
	}else {

		var colon = value.lastIndexOf(':');
		label = value.substr(colon + 1);
	}

	return label;
};

SemwidgQL.Sparql.treePathToLabelSuffix = function(node) {

	var suffix = '';

	var type = SemwidgQL.Util.getTypeLabel(node);
	switch(type) {
	case 'Resource':
	case 'Property':
	case 'PathExpression':
	case 'MultiplePathExpression':
	case 'SimpleQuery':
		suffix = '_' + node.getChildIndex();
		break;
	}

	if (!node.getParent()) {
		return suffix;
	}else {
		return SemwidgQL.Sparql.treePathToLabelSuffix(node.getParent()) + suffix;
	}

};
