SemwidgQL.Util = function () {

};

SemwidgQL.Util.truncate = function(str, maxLength, suffix) {

	if(str.length > maxLength + suffix.length + 1) {
		str = str.substring(0, maxLength + 1);
		if (str.lastIndexOf(' ') > 0) {
			str = str.substring(0, Math.min(str.length, str.lastIndexOf(' ')));
		}
		str = str + suffix;
	}

	return str;
};

SemwidgQL.Util.getTypeLabel = function(node) {
	if (!node) {
		return null;
	}

	return SemwidgQLParser.tokenNames[node.getType()];
};

SemwidgQL.Util.checkAstType = function(node, type) {

	if (!node || !type || type == '') {
		return false;
	}

	return SemwidgQL.Util.getTypeLabel(node) === type;
};

SemwidgQL.Util.getPropertyValue = function(node, removePefix) {
	if (!node) {
		return null;
	}

	if (SemwidgQL.Util.checkAstType(node, 'PrefixedProperty')){

		if (removePefix) {
			var prefix = node.getChild(0).getText().substring(0, node.getChild(0).getText().length - 1);
			var prefixList = SemwidgQL.getCurrentModel().getPrefixList();
			for (var i = 0; i < prefixList.length; i++) {
				var p = prefixList[i].getPrefix();
				if (p == prefix) {
					return '<' + prefixList[i].getUrl() + node.getChild(1).getText() + '>';
				}
			}
		}

		return node.getChild(0).getText() + node.getChild(1).getText();
	}else if (SemwidgQL.Util.checkAstType(node, 'QualifiedProperty')) {
		return '<' + node.getChild(1).getText() + '>';
	}

};

// empty expressions return "now()"
SemwidgQL.Util.readDateExpression = function(expressionString) {
	var str;

	var expression = ('' + expressionString).replace(/\s+/g, ' ').trim().split(' ');
	
	var date = 'now()';
	var operator = '-';
	var seconds = 0;

	var exp;
	
	var state = 0;

	for (var i = 0; i < expression.length; i++) {
		exp = expression[i].trim();

		if (exp != '') {
			// is date?
			if (state < 1 && exp.indexOf('-') > -1) {
				var tmp = new Date(exp);
				
				if (!isNaN(tmp.getTime())) {
					date = 'xsd:dateTime("' + tmp.toISOString() + '")';
					state = 1;
					continue;
				}
			}
			
			// is now?
			if (state < 1 && exp.toLowerCase().indexOf('now') == 0) {
				date = 'now()';
				state = 1;
				continue;
			}

			// is - or +?
			if (state < 2 && exp == '-' || exp == '+') {
				operator = exp;
				state = 2;
				continue;
			}

			// is period part
			var number = parseFloat(exp);
			if (!isNaN(number)) {
				var numberStr = '' + number;
				var unit;
				if (numberStr === exp) {

					unit = 's';
					if (expression.length > i + 1) {
						var exp2 = expression[i + 1].trim();
						if (exp2.length > 0) {
							if (SemwidgQL.Util._getMultiplicator(exp2.charAt(0).toLowerCase()) > 0) {
								unit = exp2.charAt(0).toLowerCase();
								i++;
							}
						}
					}
					
				}else if (exp.length >= numberStr.length) {
					unit = exp.charAt(numberStr.length).toLowerCase();
				}

				var mul = SemwidgQL.Util._getMultiplicator(unit);

				seconds += number * mul;

				state = 3;
			}
		}
	}

	str = date;

	if (seconds != 0) {
		str += ' ' + operator + ' ' + seconds;
	} 

	return str;
};

SemwidgQL.Util.readPeriodExpression = function(expressionString) {

	var expression = ('' + expressionString).replace(/\s+/g, ' ').trim().split(' ');

	var exp;
	var seconds = 0;

	for (var i = 0; i < expression.length; i++) {
		exp = expression[i].trim();

		if (exp != '') {

			var number = parseFloat(exp);
			if (!isNaN(number)) {
				var numberStr = '' + number;
				var unit;
				if (numberStr === exp) {

					unit = 's';
					if (expression.length > i + 1) {
						var exp2 = expression[i + 1].trim();
						if (exp2.length > 0) {
							if (SemwidgQL.Util._getMultiplicator(exp2.charAt(0).toLowerCase()) > 0) {
								unit = exp2.charAt(0).toLowerCase();
								i++;
							}
						}
					}
					
				}else if (exp.length >= numberStr.length) {
					unit = exp.charAt(numberStr.length).toLowerCase();
				}

				var mul = SemwidgQL.Util._getMultiplicator(unit);

				seconds += number * mul;
			}
		}
	}

	return seconds;
};

SemwidgQL.Util._getMultiplicator = function(str) {
	var mul = 0;
	switch(str) {
		case 's':
			mul = 1;
			break;
		case 'm':
			mul = 60;
			break;
		case 'h':
			mul = 3600;
			break;
		case 'd':
			mul = 86400;
			break;
		case 'w':
			mul = 604800;
			break;
		default:
			mul = 0;
			break;
	}
	return mul;
};